# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import MessageV2, Worklist
from driverng.fixed import FixedLengthFormat, String


class AlifaxBase(FixedLengthFormat):
    QC = False
    RESULT = False
    QUERY = False


class AlifaxQuery(AlifaxBase):
    FMT_DESCRIPTION = [
        "Q",
        String("workstation_number", 2),
        String("patient_id", 15),
        String("result", 4),
    ]
    QUERY = True


class AlifaxOrder(AlifaxBase):
    FMT_DESCRIPTION = [
        "T",
        String("workstation_number", 2),
        String("patient_id", 15),
        String("result", 4),
    ]


class AlifaxResult(AlifaxBase):
    FMT_DESCRIPTION = [
        "R",
        String("workstation_number", 2),
        String("patient_id", 15),
        String("rack", 2),
        String("position", 2),
        String("cycle", 2),
        String("result", 4),
    ]
    RESULT = True


class AlifaxQCResult(AlifaxBase):
    FMT_DESCRIPTION = [
        "R",
        String("workstation_number", 2),
        String("latex_tube", 9),
        # 1, 2 or 3
        String("level", 1),
        String("expected_value", 3),
        2,
        String("field2", 2),
        String("field3", 2),
        String("field4", 2),
        String("detected_value", 4),
    ]
    QC = True


class Driver(MessageV2):
    SPEC = {"code": "alifax", "label": "Alifax"}

    TYPE_MESSAGE_MAP = {
        "T": AlifaxOrder,
        "R": AlifaxResult,
        "Q": AlifaxQuery,
        "RQC": AlifaxQCResult,
    }

    def create(self, message_type, **kwargs):
        return self.TYPE_MESSAGE_MAP[message_type](**kwargs)

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        message_type = data[0]
        if data[0] == "R" and data[3:][:5] == "LATEX":
            message_type = "RQC"
        else:
            message_type = data[0]
        obj = self.TYPE_MESSAGE_MAP[message_type]()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) > 1:
            # FIXME
            raise RuntimeError("Multi-sample message not supported")
        elif len(samples) == 1:
            sample, _ = samples[0]
            return self.create(
                "T", workstation_number="01", patient_id=sample.sample_id, result="0001"
            )
        elif context["query"]:
            return self.create(
                "T", patient_id=context["query"].patient_id, result="0000"
            )

    def decode_message(self, message):
        if message.QUERY:
            worklist = Worklist(context="query")
            worklist.get_or_create(sample_id=message.patient_id, type="patient")
            return worklist
        worklist = Worklist(context="result")
        if message.RESULT:
            sample = worklist.get_or_create(
                sample_id=message.patient_id, type="patient"
            )
            sample.results.add(code="VS1", value=message.result)
        if message.QC:
            date = datetime.datetime.now()
            date = date.replace(second=0, microsecond=0)
            sample = worklist.get_or_create(
                control_lot=message.latex_tube + message.level,
                sample_id=message.latex_tube + message.level,
                type="control",
            )
            sample.results.add(code="VS1", value=message.detected_value, date=date)
        return worklist
